@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GtkNSSDialogs.cpp,v 1.32 2005/08/07 21:14:46 crispin Exp $
+ *  $Id: GtkNSSDialogs.cpp,v 1.41 2006/03/02 19:41:23 chpe Exp $
  */
 
 /*
@@ -31,6 +31,7 @@
 #include "config.h"
 
 #include "EphyUtils.h"
+#include "AutoJSContextStack.h"
 
 #include <nsCOMPtr.h>
 #include <nsMemory.h>
@@ -50,7 +51,7 @@
 #include <nsEmbedString.h>
 #define MOZILLA_INTERNAL_API 1
 
-#include <gconf/gconf-client.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkcheckbutton.h>
@@ -72,12 +73,13 @@
 #include <gtk/gtktextbuffer.h>
 #include <gtk/gtktextview.h>
 #include <gtk/gtkprogressbar.h>
-#include <glib/gi18n.h>
+#include <gtk/gtksizegroup.h>
+#include <glade/glade-xml.h>
+#include <gconf/gconf-client.h>
 #include <time.h>
 
 #include "GtkNSSDialogs.h"
 #include "ephy-file-helpers.h"
-#include "ephy-glade.h"
 #include "ephy-gui.h"
 
 NS_DEFINE_CID (kX509CertCID, NS_IX509CERT_IID);
@@ -96,9 +98,9 @@ GtkNSSDialogs::~GtkNSSDialogs ()
 {
 }
 
-NS_IMPL_ISUPPORTS2 (GtkNSSDialogs, 
-		    nsICertificateDialogs,
-		    nsIBadCertListener)
+NS_IMPL_THREADSAFE_ISUPPORTS2 (GtkNSSDialogs, 
+			       nsICertificateDialogs,
+			       nsIBadCertListener)
 
 /**
  *  Call the mozilla service to display a certificate
@@ -109,7 +111,7 @@ view_certificate (nsIInterfaceRequestor *ctx, nsIX509Cert *cert)
 	nsresult rv;
 	nsCOMPtr<nsICertificateDialogs> certDialogs =
 		do_GetService (NS_CERTIFICATEDIALOGS_CONTRACTID, &rv);
-	g_return_if_fail (NS_SUCCEEDED (rv));
+	NS_ENSURE_SUCCESS (rv, );
 
 	certDialogs->ViewCert (ctx, cert);
 }
@@ -178,8 +180,7 @@ higgy_setup_dialog (GtkDialog *dialog, const gchar *stock_icon,
 	gtk_widget_show (label);
 
 	/* Set up the spacing for the dialog internal widgets */
-	gtk_box_set_spacing (GTK_BOX(dialog->action_area), 6);	
-	gtk_box_set_spacing (GTK_BOX(dialog->vbox), 12);
+	gtk_box_set_spacing (GTK_BOX(dialog->vbox), 14); /* 24 = 2 * 5 + 14 */
 
 	*content_label = label;
 	if (content_vbox)
@@ -204,9 +205,17 @@ display_cert_warning_box (nsIInterfaceRequestor *ctx,
                           gboolean *checkbox_value,
 			  const char *affirmative_text)
 {
-	GtkWidget *dialog, *label, *checkbox, *vbox;
+	GtkWidget *dialog, *label, *checkbox, *vbox, *button;
 	int res;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
+	/* NOTE: Due to a mozilla bug [https://bugzilla.mozilla.org/show_bug.cgi?id=306288],
+	 * we will always end up without a parent!
+	 */
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -240,9 +249,9 @@ display_cert_warning_box (nsIInterfaceRequestor *ctx,
 		affirmative_text = _("_Accept");
 	}
 
-	gtk_dialog_add_button (GTK_DIALOG (dialog), 
-			       affirmative_text,
-			       GTK_RESPONSE_ACCEPT);
+	button = gtk_dialog_add_button (GTK_DIALOG (dialog), 
+					affirmative_text,
+					GTK_RESPONSE_ACCEPT);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 
         if (checkbox_text)
@@ -258,6 +267,9 @@ display_cert_warning_box (nsIInterfaceRequestor *ctx,
 		checkbox = 0;
 	}
 
+	/* We don't want focus on the checkbox */
+	gtk_widget_grab_focus (button);
+
 	gtk_label_set_markup (GTK_LABEL (label), markup_text);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 	gtk_widget_show_all (dialog);
@@ -301,14 +313,14 @@ GtkNSSDialogs::ConfirmMismatchDomain (nsIInterfaceRequestor *ctx,
 
 	nsEmbedCString cTargetUrl (targetURL);
 
-        first = g_markup_printf_escaped (_("The site \"%s\" returned security information for "
-					   "\"%s\". It is possible that someone is intercepting "
+	first = g_markup_printf_escaped (_("The site “%s” returned security information for "
+					   "“%s”. It is possible that someone is intercepting "
 					   "your communication to obtain your confidential "
 					   "information."),
 					 cTargetUrl.get(), cCommonName.get());
 
         second = g_markup_printf_escaped (_("You should only accept the security information if you "
-					    "trust \"%s\" and \"%s\"."),
+					    "trust “%s” and “%s”."),
 					  cTargetUrl.get(), cCommonName.get());
 	
 	msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n\n%s",
@@ -343,14 +355,14 @@ GtkNSSDialogs::ConfirmUnknownIssuer (nsIInterfaceRequestor *ctx,
 			   NS_CSTRING_ENCODING_UTF8, cCommonName);
 
 	secondary = g_markup_printf_escaped
-		           (_("It was not possible to automatically trust \"%s\". "
+		           (_("It was not possible to automatically trust “%s”. "
 			      "It is possible that someone is intercepting your "
 			      "communication to obtain your confidential information."),
 			      cCommonName.get());
 
         tertiary = g_markup_printf_escaped
 		           (_("You should only connect to the site if you are certain "
-			      "you are connected to \"%s\"."),
+			      "you are connected to “%s”."),
 			    cCommonName.get());
 
 	msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n\n%s",
@@ -419,14 +431,16 @@ GtkNSSDialogs::ConfirmCertExpired (nsIInterfaceRequestor *ctx,
 	if (LL_CMP(now, >, notAfter))
 	{
 		primary = _("Accept expired security information?");
-		text    = _("The security information for \"%s\" "
+		/* Translators: first %s is a hostname, second %s is a time/date */
+		text    = _("The security information for “%s” "
 			    "expired on %s.");
 		timeToUse = notAfter;
 	} 
 	else
 	{
 		primary = _("Accept not yet valid security information?");
-		text    = _("The security information for \"%s\" isn't valid until %s.");
+		/* Translators: first %s is a hostname, second %s is a time/date */
+		text    = _("The security information for “%s” isn't valid until %s.");
 		timeToUse = notBefore;
 	}
 	
@@ -496,10 +510,10 @@ GtkNSSDialogs::NotifyCrlNextupdate (nsIInterfaceRequestor *ctx,
 
 	nsEmbedCString cTargetUrl (targetURL);
 
-	primary = g_markup_printf_escaped (_("Cannot establish connection to \"%s\"."),
+	primary = g_markup_printf_escaped (_("Cannot establish connection to “%s”."),
 					   cTargetUrl.get());
 
-	secondary = g_markup_printf_escaped (_("The certificate revocation list (CRL) from \"%s\" "
+	secondary = g_markup_printf_escaped (_("The certificate revocation list (CRL) from “%s” "
 					       "needs to be updated."),
 					     cCommonName.get());
 	msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n\n%s",
@@ -531,6 +545,11 @@ GtkNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *label;
 	char *msg, *primary;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -561,7 +580,7 @@ GtkNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx,
 	NS_UTF16ToCString (commonName,
 			   NS_CSTRING_ENCODING_UTF8, cCommonName);
 
-	primary = g_markup_printf_escaped (_("Trust new Certificate Authority \"%s\" to identify web sites?"),
+	primary = g_markup_printf_escaped (_("Trust new Certificate Authority “%s” to identify web sites?"),
 					   cCommonName.get());
 
 	msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
@@ -743,6 +762,11 @@ GtkNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	GtkWidget *progress;
 	char *msg;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -763,7 +787,8 @@ GtkNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	higgy_setup_dialog (GTK_DIALOG (dialog), GTK_STOCK_DIALOG_QUESTION,
 			    &label, &vbox);
 
-	button = gtk_button_new_with_mnemonic (_("_Backup Certificate"));
+	/* Translators: this is the action of the certificate being exported to a backup file */
+	button = gtk_button_new_with_mnemonic (_("_Back Up Certificate"));
 	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
 	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
@@ -777,7 +802,7 @@ GtkNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 
 	table = gtk_table_new (3, 3, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
 	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
 
 	label = gtk_label_new (NULL);
@@ -860,6 +885,11 @@ GtkNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *hbox, *label, *entry, *vbox;
 	char *msg;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -923,22 +953,35 @@ GtkNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 
 
 static void
-set_table_row (GtkWidget *table, int row, const char *title, GtkWidget *label)
+set_table_row (GtkWidget *table,
+	       int& row,
+	       const char *title,
+	       const char *text)
 {
-	GtkWidget *header;
-	char buf[64];
+	GtkWidget *header, *label;
+	char *bold;
 
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", title);
-	header = gtk_label_new (buf);
-	gtk_label_set_use_markup (GTK_LABEL(header), TRUE);
-	gtk_misc_set_alignment (GTK_MISC(header), 0, 0);
+	if (text == NULL || text[0] == 0) return;
+
+	bold = g_markup_printf_escaped ("<b>%s</b>", title);
+	header = gtk_label_new (bold);
+	g_free (bold);
+
+	gtk_label_set_use_markup (GTK_LABEL (header), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (header), 0, 0);
 	gtk_widget_show (header);
 	gtk_table_attach (GTK_TABLE (table), header, 0, 1, row, row+1,
 			  GTK_FILL, GTK_FILL, 0, 0);
-	
-	gtk_misc_set_alignment (GTK_MISC(label), 0, 0);
+
+	label = gtk_label_new (text);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+	gtk_label_set_max_width_chars (GTK_LABEL (label), 48);
 	gtk_widget_show (label);
 	gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, row, row+1);
+
+	row++;
 }
 
 NS_IMETHODIMP 
@@ -959,19 +1002,23 @@ GtkNSSDialogs::CrlImportStatusDialog(nsIInterfaceRequestor *ctx, nsICRLInfo *crl
 					      NULL);
 
 	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Certificate Revocation List Imported"));
+
+	/* Needed because gparent == NULL always because of mozilla sucks */
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (dialog), TRUE);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), TRUE);
 
 	higgy_setup_dialog (GTK_DIALOG (dialog), GTK_STOCK_DIALOG_INFO,
 			    &label, &vbox);
 
-	msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
-			       _("Certificate Revocation list successfully imported."),
-			       _("Certificate Revocation list (CRL) imported:"));
+	msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>",
+			       _("Certificate Revocation List (CRL) successfully imported"));
 	gtk_label_set_markup (GTK_LABEL (label), msg);
 	g_free (msg);
 
 	table = gtk_table_new (2, 3, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
 
 	nsEmbedString org, orgUnit, nextUpdate;
 	rv = crl->GetOrganization (org);
@@ -983,22 +1030,20 @@ GtkNSSDialogs::CrlImportStatusDialog(nsIInterfaceRequestor *ctx, nsICRLInfo *crl
 	rv = crl->GetNextUpdateLocale (nextUpdate);
 	if (NS_FAILED(rv)) return rv;
 
+	int row = 0;
 	nsEmbedCString cOrg;
 	NS_UTF16ToCString (org, NS_CSTRING_ENCODING_UTF8, cOrg);
-	label = gtk_label_new (cOrg.get());
-	set_table_row (table, 0, _("Organization:"), label);
+	set_table_row (table, row, _("Organization:"), cOrg.get ());
 
 	nsEmbedCString cOrgUnit;
 	NS_UTF16ToCString (orgUnit, NS_CSTRING_ENCODING_UTF8, cOrgUnit);
-	label = gtk_label_new (cOrgUnit.get());
-	set_table_row (table, 1, _("Unit:"), label);
+	set_table_row (table, row, _("Unit:"), cOrgUnit.get ());
 
 	nsEmbedCString cNextUpdate;
 	NS_UTF16ToCString (nextUpdate, NS_CSTRING_ENCODING_UTF8, cNextUpdate);
-	label = gtk_label_new (cNextUpdate.get());
-	set_table_row (table, 2, _("Next Update:"), label);
+	set_table_row (table, row, _("Next Update:"), cNextUpdate.get ());
 
-	gtk_box_pack_start (GTK_BOX (vbox), higgy_indent_widget (table), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
 
 	gtk_widget_show_all (dialog);
 	g_signal_connect (G_OBJECT (dialog),
@@ -1091,10 +1136,11 @@ fill_cert_chain_tree (GtkTreeView *treeview, nsIArray *certChain)
 	}
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (treeview));
 
-	/* And select the last entry */
+	/* And select the last entry, and scroll the view so it's visible */
 	GtkTreeSelection *select = gtk_tree_view_get_selection (treeview);
 	GtkTreePath *path = gtk_tree_model_get_path (model, &parent);
 	gtk_tree_selection_select_path (select, path);
+	gtk_tree_view_scroll_to_cell (treeview, path, NULL, TRUE, 0.5, 0.0);
 	gtk_tree_path_free (path);
 
 	return TRUE; 
@@ -1288,14 +1334,21 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *widget;
 	GladeXML *gxml;
 	nsEmbedString value;
-	nsresult rv;
 	PRUint32 verifystate, count;
 	PRUnichar ** usage;
 	GtkSizeGroup * sizegroup;
 
-	gxml = ephy_glade_widget_new (ephy_file ("certificate-dialogs.glade"),
-				      "viewcert_dialog",
-				      &dialog, NULL, NULL);
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
+	gxml = glade_xml_new (ephy_file ("certificate-dialogs.glade"),
+			      "viewcert_dialog", NULL);
+	g_return_val_if_fail (gxml != NULL, NS_ERROR_FAILURE);
+
+	dialog = glade_xml_get_widget (gxml, "viewcert_dialog");
+	g_return_val_if_fail (dialog != NULL, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
