@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: ephy-bookmarks-editor.c,v 1.177 2005/10/16 20:29:26 chpe Exp $
+ * $Id: ephy-bookmarks-editor.c,v 1.192 2006/03/10 09:21:06 chpe Exp $
  */
 
 #include "config.h"
@@ -48,6 +48,9 @@
 #include "ephy-bookmarks-editor.h"
 #include "ephy-bookmarks-import.h"
 #include "ephy-bookmarks-export.h"
+#include "ephy-bookmarks-ui.h"
+#include "ephy-bookmark-action.h"
+#include "ephy-topic-action.h"
 #include "ephy-node-common.h"
 #include "ephy-node-view.h"
 #include "ephy-window.h"
@@ -67,26 +70,34 @@
 #include "ephy-debug.h"
 #include "egg-toolbars-model.h"
 
-static GtkTargetEntry topic_drag_dest_types [] =
+static const GtkTargetEntry topic_drag_dest_types [] =
 {
 	{ EPHY_DND_URI_LIST_TYPE,   0, 0 }
 };
 
-static int n_topic_drag_dest_types = G_N_ELEMENTS (topic_drag_dest_types);
-
-static GtkTargetEntry bmk_drag_types [] =
+static const GtkTargetEntry bmk_drag_types [] =
 {
 	{ EPHY_DND_URL_TYPE,        0, 0 },
 	{ EPHY_DND_URI_LIST_TYPE,   0, 1 },
 	{ EPHY_DND_TEXT_TYPE,       0, 2 }
 };
-static int n_bmk_drag_types = G_N_ELEMENTS (bmk_drag_types);
 
-static GtkTargetEntry topic_drag_types [] =
+static const GtkTargetEntry topic_drag_types [] =
 {
 	{ EPHY_DND_TOPIC_TYPE,      0, 0 }
 };
-static int n_topic_drag_types = G_N_ELEMENTS (topic_drag_types);
+
+static const struct
+{
+	const char *name;
+	const char *extension;
+}
+export_formats [] = 
+/* Don't add or reoder those entries without updating export_dialog_response_cb too! */
+{
+	{ N_("Epiphany (RDF)"), "rdf" },
+	{ N_("Mozilla (HTML)"), "html" }
+};
 
 static void ephy_bookmarks_editor_class_init (EphyBookmarksEditorClass *klass);
 static void ephy_bookmarks_editor_init (EphyBookmarksEditor *editor);
@@ -106,6 +117,8 @@ static void cmd_open_bookmarks_in_tabs    (GtkAction *action,
 					   EphyBookmarksEditor *editor);
 static void cmd_open_bookmarks_in_browser (GtkAction *action,
 					   EphyBookmarksEditor *editor);
+static void cmd_toolbar			  (GtkAction *action,
+					   EphyBookmarksEditor *editor);
 static void cmd_delete			  (GtkAction *action,
 					   EphyBookmarksEditor *editor);
 static void cmd_bookmark_properties	  (GtkAction *action,
@@ -162,7 +175,7 @@ enum
 
 static GObjectClass *parent_class = NULL;
 
-static GtkActionEntry ephy_bookmark_popup_entries [] = {
+static const GtkActionEntry ephy_bookmark_popup_entries [] = {
 	/* Toplevel */
 	{ "File", NULL, N_("_File") },
 	{ "Edit", NULL, N_("_Edit") },
@@ -220,7 +233,12 @@ static GtkActionEntry ephy_bookmark_popup_entries [] = {
 	  N_("Display credits for the web browser creators"),
 	  G_CALLBACK (window_cmd_help_about) },
 };
-static guint ephy_bookmark_popup_n_entries = G_N_ELEMENTS (ephy_bookmark_popup_entries);
+
+static const GtkToggleActionEntry ephy_bookmark_toggle_entries [] = {
+	{ "ShowOnToolbar", NULL, N_("_Show on Toolbar"), NULL,
+	  N_("Show the selected bookmark on a toolbar"), 
+	  G_CALLBACK (cmd_toolbar), FALSE },
+};
 
 enum
 {
@@ -228,7 +246,7 @@ enum
 	VIEW_TITLE_AND_ADDRESS
 };
 
-static GtkRadioActionEntry ephy_bookmark_radio_entries [] =
+static const GtkRadioActionEntry ephy_bookmark_radio_entries [] =
 {
 	/* View Menu */
 	{ "ViewTitle", NULL, N_("_Title"), NULL,
@@ -237,7 +255,6 @@ static GtkRadioActionEntry ephy_bookmark_radio_entries [] =
 	  N_("Show both the title and address columns"),
 	  VIEW_TITLE_AND_ADDRESS } 
 };
-static guint ephy_bookmark_n_radio_entries = G_N_ELEMENTS (ephy_bookmark_radio_entries);
 
 static void
 entry_selection_changed_cb (GtkWidget *widget, GParamSpec *pspec, EphyBookmarksEditor *editor)
@@ -394,7 +411,7 @@ delete_topic_dialog_construct (GtkWindow *parent,
 					 GTK_DIALOG_MODAL,
 					 GTK_MESSAGE_WARNING,
 					 GTK_BUTTONS_CANCEL,
-					 _("Delete topic \"%s\"?"),
+					 _("Delete topic “%s”?"),
 					 topic);
 	
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Delete this topic?"));
@@ -410,6 +427,118 @@ delete_topic_dialog_construct (GtkWindow *parent,
 	return dialog;
 }
 
+static gint
+get_bookmarks_bar (EggToolbarsModel *model)
+{
+	gint tpos;
+	const char *tname;
+	
+	for (tpos = 0; tpos < egg_toolbars_model_n_toolbars (model); tpos++)
+	{
+		tname = egg_toolbars_model_toolbar_nth (model, tpos);
+
+		if (tname != NULL &&
+		    strcmp (tname, "BookmarksBar") == 0)
+			break;
+	}
+	
+	if (tpos == egg_toolbars_model_n_toolbars (model))
+	{		
+		tpos = egg_toolbars_model_add_toolbar (model, -1, "BookmarksBar");
+	}
+	
+	return tpos;
+}
+
+static void
+cmd_toolbar (GtkAction *action,
+	     EphyBookmarksEditor *editor)
+{
+	EggToolbarsModel *model;
+	EphyNode *node;
+	gboolean show;
+	gint flags, tpos = 0;
+	GList *selection;
+	GList *l;
+	
+	model = EGG_TOOLBARS_MODEL (ephy_shell_get_toolbars_model (ephy_shell_get_default (), FALSE));
+		
+	if (ephy_node_view_is_target (EPHY_NODE_VIEW (editor->priv->bm_view)))
+	{
+		char name[EPHY_BOOKMARK_ACTION_NAME_BUFFER_SIZE];
+
+		selection = ephy_node_view_get_selection (EPHY_NODE_VIEW (editor->priv->bm_view));
+		
+		node = selection->data;
+
+		EPHY_BOOKMARK_ACTION_NAME_PRINTF (name, node);
+
+		flags = egg_toolbars_model_get_name_flags (model, name);
+		show = ((flags & EGG_TB_MODEL_NAME_USED) == 0);
+		
+		if (show)
+		{
+			tpos = get_bookmarks_bar (model);
+		}
+
+		for (l = selection; l; l = l->next)
+		{
+			node = l->data;
+
+			EPHY_BOOKMARK_ACTION_NAME_PRINTF (name, node);
+
+			flags = egg_toolbars_model_get_name_flags (model, name);
+			if(show && ((flags & EGG_TB_MODEL_NAME_USED) == 0))
+			{
+				egg_toolbars_model_add_item (model, tpos, -1, name);
+			}
+			else if(!show && ((flags & EGG_TB_MODEL_NAME_USED) != 0))
+			{
+				egg_toolbars_model_delete_item (model, name);
+			}
+		}
+
+		g_list_free (selection);
+	}
+	else if (ephy_node_view_is_target (EPHY_NODE_VIEW (editor->priv->key_view)))
+	{
+		char name[EPHY_TOPIC_ACTION_NAME_BUFFER_SIZE];
+
+		selection = ephy_node_view_get_selection (EPHY_NODE_VIEW (editor->priv->key_view));
+	  
+		node = selection->data;
+
+		EPHY_TOPIC_ACTION_NAME_PRINTF (name, node);
+
+		flags = egg_toolbars_model_get_name_flags (model, name);
+		show = ((flags & EGG_TB_MODEL_NAME_USED) == 0);
+
+		if (show)
+		{
+			tpos = get_bookmarks_bar (model);
+		}
+
+		for (l = selection; l; l = l->next)
+		{
+			node = l->data;
+
+			EPHY_TOPIC_ACTION_NAME_PRINTF (name, node);
+
+			flags = egg_toolbars_model_get_name_flags (model, name);
+			if(show && ((flags & EGG_TB_MODEL_NAME_USED) == 0))
+			{
+				egg_toolbars_model_add_item (model, tpos, -1, name);
+			}
+			else if(!show && ((flags & EGG_TB_MODEL_NAME_USED) != 0))
+			{
+				egg_toolbars_model_delete_item (model, name);
+			}
+		}
+
+		g_list_free (selection);
+	}
+}
+
 static void
 cmd_delete (GtkAction *action,
 	    EphyBookmarksEditor *editor)
@@ -463,14 +592,6 @@ cmd_delete (GtkAction *action,
 	}
 }
 
-static void
-show_properties_dialog (EphyBookmarksEditor *editor,
-			EphyNode *bookmark)
-{
-	ephy_bookmarks_show_bookmark_properties
-		(editor->priv->bookmarks, bookmark, GTK_WIDGET (editor));
-}
-
 static GSList *
 add_bookmarks_files (const char *dir,
 		     const char *filename,
@@ -508,7 +629,23 @@ add_bookmarks_source (const char *file,
 
 		if (strcmp (p, "firefox") == 0)
 		{
-			description = g_strdup (_("Firefox"));
+			const char *profile = NULL, *dot;
+
+			if (path[i+1] != NULL)
+			{
+				dot = strchr (path[i+1], '.');
+				profile = dot ? dot + 1 : path[i+1];
+			}
+
+			if (profile != NULL && strcmp (profile, "default") != 0)
+			{
+				/* FIXME: proper i18n after freeze */
+				description = g_strdup_printf ("%s “%s”", _("Firefox"), profile);
+			}
+			else
+			{
+				description = g_strdup (_("Firefox"));
+			}
 		}
 		else if (strcmp (p, ".firefox") == 0)
 		{
@@ -521,7 +658,7 @@ add_bookmarks_source (const char *file,
 		else if (strcmp (p, ".mozilla") == 0)
 		{
 			/* Translators: The %s is the name of a Mozilla profile. */
-			description = g_strdup_printf (_("Mozilla \"%s\" profile"), path[i+1]);
+			description = g_strdup_printf (_("Mozilla “%s” profile"), path[i+1]);
 		}
 		else if (strcmp (p, ".galeon") == 0)
 		{
@@ -563,7 +700,7 @@ import_bookmarks (EphyBookmarksEditor *editor,
 		gtk_window_set_title (GTK_WINDOW (dialog), _("Import Failed"));
 		gtk_message_dialog_format_secondary_text
 			(GTK_MESSAGE_DIALOG (dialog),
-			 _("The bookmarks from \"%s\" could not be imported "
+			 _("The bookmarks from “%s” could not be imported "
 			   "because the file is corrupted or of an "
 			   "unsupported type."),
 			 basename);
@@ -604,7 +741,7 @@ import_from_file_response_cb (GtkWidget *dialog,
 
 static void
 import_dialog_response_cb (GtkDialog *dialog,
-			   gint response,
+			   int response,
 			   EphyBookmarksEditor *editor)
 {
 	if (response == GTK_RESPONSE_OK)
@@ -667,13 +804,102 @@ import_dialog_response_cb (GtkDialog *dialog,
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
+static void
+export_format_combo_changed_cb (GtkComboBox *combo,
+				GtkFileChooser *chooser)
+{
+	char *filename, *basename, *dot, *newname;
+	int i, format;
+
+	filename = gtk_file_chooser_get_filename (chooser);
+	if (filename == NULL) return;
+
+	basename = g_path_get_basename (filename);
+	if (basename == NULL || basename[0] == '\0')
+	{
+		g_free (filename);
+		g_free (basename);
+		return;
+	}
+
+	format = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
+	g_return_if_fail (format >= 0 && format < G_N_ELEMENTS (export_formats));
+
+	dot = strrchr (basename, '.');
+	if (dot != NULL)
+	{
+		for (i = 0; i < G_N_ELEMENTS (export_formats); ++i)
+		{
+			if (strcmp (dot + 1, export_formats[i].extension) == 0)
+			{
+				*dot = '\0';
+				break;
+			}
+		}
+	}
+
+	newname = g_strconcat (basename, ".",
+			       export_formats[format].extension,
+			       NULL);
+
+	gtk_file_chooser_set_current_name (chooser, newname);
+
+	g_free (filename);
+	g_free (basename);
+	g_free (newname);
+}
+
+static void
+export_dialog_response_cb (GtkWidget *dialog,
+			   int response,
+			   EphyBookmarksEditor *editor)
+{
+	GtkWidget *combo;
+	char *filename;
+	int format;
+
+	if (response != GTK_RESPONSE_ACCEPT)
+	{
+		gtk_widget_destroy (dialog);
+		return;
+	}
+
+	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+	if (filename == NULL) return;
+	
+	if (!ephy_gui_check_location_writable (GTK_WIDGET (dialog), filename))
+	{
+		g_free (filename);
+		return;
+	}
+
+	combo = g_object_get_data (G_OBJECT (dialog), "format-combo");
+	g_return_if_fail (combo != NULL);
+
+	format = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
+	g_return_if_fail (format >= 0 && format < G_N_ELEMENTS (export_formats));
+
+	gtk_widget_destroy (dialog);
+
+	/* 0 for ephy RDF format, 1 for mozilla HTML format */
+	if (format == 0)
+	{
+		ephy_bookmarks_export_rdf (editor->priv->bookmarks, filename);
+	}
+	else if (format == 1)
+	{
+		ephy_bookmarks_export_mozilla (editor->priv->bookmarks, filename);
+	}
+
+	g_free (filename);
+}
+
 static void
 cmd_bookmarks_export (GtkAction *action,
 		      EphyBookmarksEditor *editor)
 {
 	GtkWidget *dialog, *hbox, *label, *combo;
-	char *filename = NULL;
-	int response, format;
+	int format;
 
 	dialog = GTK_WIDGET (ephy_file_chooser_new (_("Export Bookmarks"),
 		GTK_WIDGET (editor),
@@ -693,10 +919,18 @@ cmd_bookmarks_export (GtkAction *action,
 	label = gtk_label_new (_("File format:"));
 
 	combo = gtk_combo_box_new_text ();
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Epiphany (RDF)"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Mozilla (HTML)"));
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 1);
-	
+
+	for (format = 0; format < G_N_ELEMENTS (export_formats); ++format)
+	{
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
+					   _(export_formats[format].name));
+	}
+
+	g_object_set_data (G_OBJECT (dialog), "format-combo", combo);
+	g_signal_connect (combo, "changed",
+			  G_CALLBACK (export_format_combo_changed_cb), dialog);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
+
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_end (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
 	gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -706,50 +940,9 @@ cmd_bookmarks_export (GtkAction *action,
 
 	gtk_window_group_add_window (GTK_WINDOW (editor)->group, GTK_WINDOW (dialog));
 
-	do
-	{
-		char *basename, *strtmp = NULL;
-
-		g_free (filename);
-
-		response = gtk_dialog_run (GTK_DIALOG (dialog));
-
-		format = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
-		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-		basename = g_path_get_basename (filename);
-		if (basename != NULL && strchr (basename, '.') == NULL)
-		{
-			if (format == 0)
-			{
-				strtmp = filename;
-				filename = g_strconcat (filename, ".rdf", NULL);
-			}
-			else if (format == 1)
-			{
-				strtmp = filename;
-				filename = g_strconcat (filename, ".html", NULL);
-			}
-			g_free (strtmp);
-		}
-		g_free (basename);
-	}
-	while (response == GTK_RESPONSE_ACCEPT
-	       && !ephy_gui_check_location_writable (GTK_WIDGET (dialog), filename));
-
-	gtk_widget_destroy (dialog);
-
-	/* 0 for ephy RDF format, 1 for mozilla HTML format */
-
-	if (format == 0)
-	{
-		ephy_bookmarks_export_rdf (editor->priv->bookmarks, filename);
-	}
-	else if (format == 1)
-	{
-		ephy_bookmarks_export_mozilla (editor->priv->bookmarks, filename);
-	}
-
-	g_free (filename);
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (export_dialog_response_cb), editor);
+	gtk_widget_show (dialog);
 }
 
 static void
@@ -848,7 +1041,8 @@ cmd_bookmark_properties (GtkAction *action,
 	for (l = selection; l; l = l->next)
 	{
 		EphyNode *node = l->data;
-		show_properties_dialog (editor, node);
+
+		ephy_bookmarks_ui_show_bookmark (node);
 	}
 
 	g_list_free (selection);
@@ -1040,7 +1234,7 @@ ephy_bookmarks_editor_class_init (EphyBookmarksEditorClass *klass)
 							      "Bookmarks set",
 							      "Bookmarks set",
 							      EPHY_TYPE_BOOKMARKS,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (object_class, sizeof(EphyBookmarksEditorPrivate));
 }
@@ -1057,9 +1251,10 @@ ephy_bookmarks_editor_finalize (GObject *object)
 
 	if (editor->priv->window)
 	{
+		GtkWidget **window = &editor->priv->window;
 		g_object_remove_weak_pointer
 			(G_OBJECT(editor->priv->window),
-			 (gpointer *)&editor->priv->window);
+			 (gpointer *)window);
 	}
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -1091,7 +1286,10 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	gboolean key_normal = FALSE;
 	gboolean cut, copy, paste, select_all;
 	gboolean mutable = TRUE;
+	gboolean showtoolbar = FALSE;
+	gboolean ontoolbar = FALSE;
 
+	EggToolbarsModel *model;
 	GtkActionGroup *action_group;
 	GtkAction *action;
 	GList *selected;
@@ -1100,6 +1298,9 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 
 	LOG ("Update menu sensitivity");
 
+	model = EGG_TOOLBARS_MODEL (ephy_shell_get_toolbars_model 
+				    (ephy_shell_get_default(), FALSE));
+	
 	bmk_focus = ephy_node_view_is_target
 		(EPHY_NODE_VIEW (editor->priv->bm_view));
 	key_focus = ephy_node_view_is_target
@@ -1139,12 +1340,18 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	{
 		EphyNode *node = selected->data;
 		EphyNodePriority priority;
+		char name[EPHY_TOPIC_ACTION_NAME_BUFFER_SIZE];
 
 		priority = ephy_node_get_property_int
 			(node, EPHY_NODE_KEYWORD_PROP_PRIORITY);
 		if (priority == -1) priority = EPHY_NODE_NORMAL_PRIORITY;
 		key_normal = (priority == EPHY_NODE_NORMAL_PRIORITY);
 
+		EPHY_TOPIC_ACTION_NAME_PRINTF (name, node);
+
+		ontoolbar = ((egg_toolbars_model_get_name_flags (model, name)
+			      & EGG_TB_MODEL_NAME_USED) != 0);
+
 		g_list_free (selected);
 	}
 
@@ -1152,12 +1359,16 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	if (bmk_focus && selected)
 	{
 		EphyNode *node = selected->data;
-		guint id;
+		char name[EPHY_BOOKMARK_ACTION_NAME_BUFFER_SIZE];
 
 		g_return_if_fail (node != NULL);
 
-		id = ephy_node_get_id (node);
 		mutable = !ephy_node_get_property_boolean (node, EPHY_NODE_BMK_PROP_IMMUTABLE);
+		
+		EPHY_BOOKMARK_ACTION_NAME_PRINTF (name, node);
+
+		ontoolbar = ((egg_toolbars_model_get_name_flags (model, name)
+			      & EGG_TB_MODEL_NAME_USED) != 0);
 
 		g_list_free (selected);
 	}
@@ -1185,6 +1396,8 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	delete = (bmk_focus && bmk_selection && mutable) ||
 		 (key_selection && key_focus && key_normal);
 	properties = bmk_focus && single_bmk_selected && mutable;
+	showtoolbar = (bmk_focus && bmk_selection) ||
+	              (key_focus && key_selection);
 
 	action_group = editor->priv->action_group;
 	action = gtk_action_group_get_action (action_group, "OpenInWindow");
@@ -1208,6 +1421,11 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	gtk_action_set_sensitive (action, paste);
 	action = gtk_action_group_get_action (action_group, "SelectAll");
 	g_object_set (action, "sensitive", select_all, NULL);
+	action = gtk_action_group_get_action (action_group, "ShowOnToolbar");
+	gtk_action_set_sensitive (action, showtoolbar);
+	g_signal_handlers_block_by_func (action, cmd_toolbar, editor);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), ontoolbar);
+	g_signal_handlers_unblock_by_func (action, cmd_toolbar, editor);
 }
 
 static gboolean
@@ -1553,12 +1771,14 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	action_group = gtk_action_group_new ("PopupActions");
 	gtk_action_group_set_translation_domain (action_group, NULL);
 	gtk_action_group_add_actions (action_group, ephy_bookmark_popup_entries,
-				      ephy_bookmark_popup_n_entries, editor);
+				      G_N_ELEMENTS (ephy_bookmark_popup_entries), editor);
+	gtk_action_group_add_toggle_actions (action_group, ephy_bookmark_toggle_entries,
+					     G_N_ELEMENTS (ephy_bookmark_toggle_entries), editor);
 
 	details_value = get_details_value (editor);
 	gtk_action_group_add_radio_actions (action_group,
 					    ephy_bookmark_radio_entries,
-					    ephy_bookmark_n_radio_entries,
+					    G_N_ELEMENTS (ephy_bookmark_radio_entries),
 					    details_value,
 					    G_CALLBACK (cmd_view_columns), 
 					    editor);
@@ -1608,11 +1828,11 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 				   EPHY_NODE_VIEW_SEARCHABLE, NULL, NULL);
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (key_view),
 					   topic_drag_types,
-					   n_topic_drag_types,
+					   G_N_ELEMENTS (topic_drag_types),
 					   col_id, -1);
 	ephy_node_view_enable_drag_dest (EPHY_NODE_VIEW (key_view),
 					 topic_drag_dest_types,
-					 n_topic_drag_dest_types);
+					 G_N_ELEMENTS (topic_drag_dest_types));
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (key_view));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
 	g_signal_connect (G_OBJECT (selection),
@@ -1681,7 +1901,7 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 				   0, NULL, &(editor->priv->address_col));
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (bm_view),
 					   bmk_drag_types,
-					   n_bmk_drag_types,
+					   G_N_ELEMENTS (bmk_drag_types),
 					   url_col_id, title_col_id);
 	ephy_node_view_set_sort (EPHY_NODE_VIEW (bm_view), G_TYPE_STRING,
 				 EPHY_NODE_BMK_PROP_TITLE, GTK_SORT_ASCENDING);
@@ -1721,18 +1941,21 @@ void
 ephy_bookmarks_editor_set_parent (EphyBookmarksEditor *ebe,
 				  GtkWidget *window)
 {
+	GtkWidget **w;
 	if (ebe->priv->window)
 	{
+		w = &ebe->priv->window;
 		g_object_remove_weak_pointer
 			(G_OBJECT(ebe->priv->window),
-			 (gpointer *)&ebe->priv->window);
+			 (gpointer *)w);
 	}
 
 	ebe->priv->window = window;
+	w = &ebe->priv->window;
 
 	g_object_add_weak_pointer
 			(G_OBJECT(ebe->priv->window),
-			 (gpointer *)&ebe->priv->window);
+			 (gpointer *)w);
 
 }
 
