@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-toolbar-editor.c,v 1.6 2005/11/08 12:56:23 chpe Exp $
+ *  $Id: ephy-toolbar-editor.c,v 1.7 2006/01/25 23:33:05 chpe Exp $
  */
 
 #include "config.h"
@@ -199,10 +199,13 @@ ephy_toolbar_editor_constructor (GType type,
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Toolbar Editor"));
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (priv->window));
-	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
 	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
 
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (priv->window));
+	gtk_window_group_add_window (ephy_gui_ensure_window_group (GTK_WINDOW (priv->window)),
+				     GTK_WINDOW (dialog));
+	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+
 	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (priv->window));
 	editor = egg_toolbar_editor_new (manager, priv->model);
 	gtk_container_set_border_width (GTK_CONTAINER (EGG_TOOLBAR_EDITOR (editor)), 5);
